//Copyright (C) 2011, FIE LLC.

//See LICENSE.TXT for details.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows.Forms;
using libcontextutilsclient;
using libscclient;

namespace scgui
{
    public partial class SCGuiWindow : Form
    {

        libscclient.ISCUIAppContextClient scguiContext;
        bool branchingAllowed = true;

        public libscclient.ISCUIAppContextClient SCGuiContext
        {
            get { return scguiContext; }
            set { scguiContext = value; }
        }

        private libscclient.LaunchableContextSettingsPlatform launchers;
		private libscclient.IPedestalClient pedestal;
		private libscclient.Client client;

        private TreeNode CreateNode(TreeNode root, string path)
        {
            TreeNode current = root.Nodes[path];
            if (current == null)
            {
                current = new TreeNode(path);
                current.Name = path;
                root.Nodes.Add(current);
            }
            return current;
        }

        private void AddNodeAtPath(TreeNode parentNode, string newNodeName, string[] path)
        {
			System.Console.WriteLine("Adding Node at Path: name: " + newNodeName);
            TreeNode currentNode = parentNode;
            
            if (path != null)
            {
                foreach (string s in path)
                {
                    currentNode = CreateNode(currentNode, s);                    
                }
            }
            TreeNode nodeToAdd = new ContextTreeNode(newNodeName);
            nodeToAdd.Name = newNodeName;
            currentNode.Nodes.Add(nodeToAdd);
        }

        private void BuildTree()
        {
            string currentNamespace = null;
            string reason = string.Empty;
			
			libscclient.IContextClient[] stack = client.GetStack();
			
            if (stack.Length > 1)
            {
				currentNamespace = stack[stack.Length - 2].GetSettings().NameSpace;
            }
            TreeNode nsRootNode = null;

            foreach (string ns in launchers.ListNamespaces())
            {
                // Create namespace root node.
                nsRootNode = new NamespaceTreeNode(ns);
                nsRootNode.Name = ns;
                this.treeView1.Nodes.Add(nsRootNode);

                // Iterate through launchers for this ns
				foreach (string name in launchers.ListSettingsForNamespace(ns)){
					libscclient.LaunchableContextSettings launchableSettings = launchers.Find(ns,name);
					if (launchableSettings.CanRun){
                        // create node
                        AddNodeAtPath(nsRootNode, name, launchableSettings.Path);
                    }
                    else
                    {
                        System.Console.WriteLine("context cannot run: " + launchableSettings.Reason);
                    }
                }
                if (ns == currentNamespace || currentNamespace == null)
                {
                    nsRootNode.Expand();
                }
                else
                {
                    nsRootNode.Collapse();
                }
            }
        }

        public SCGuiWindow(libscclient.ISCUIAppContextClient currentContext, bool allowBranching, libscclient.Client client, libscclient.IPedestalClient pedestal)
        {
            this.scguiContext = currentContext;
            this.branchingAllowed = allowBranching;
			this.pedestal = pedestal;
			this.client = client;
            this.launchers = client.GetLaunchPlatformSettings();
            InitializeComponent();
            BuildTree();
			string stackString = "context:";
			var stack = client.GetStack();
			foreach(libscclient.IContextClient context in stack){
				var settings = context.GetSettings();
				if (settings is libscclient.SCGUIAppContextSettings){
					continue;
				}
				stackString += "/";
				//stackString += settings.NameSpace;
				//stackString += ":";
				stackString += settings.Name;
			}
			this.label1.Text = stackString;
        }

        private void nodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            TreeNode selectedNode = e.Node;
            if (selectedNode is ContextTreeNode)
            {
                string name = selectedNode.Name;
                TreeNode currentNode = selectedNode.Parent;
                while (currentNode.Parent != null)
                {
                    currentNode = currentNode.Parent;
                }
                string ns = currentNode.Name;
				libscclient.LaunchableContextSettings launchableSettings;
				launchableSettings = this.launchers.Find(ns,name);

                //branch based on modifier keys
                if (Control.ModifierKeys == Keys.Shift)
                {
                    //shift key pressed.
                    //this means leave this SCGui up and make a new on with the selected context.
                    //does nothing if branching is not allowed.

                    libscclient.ContextSettings settings = launchableSettings.Settings;
                    if (launchableSettings.IsIntermediate)
                    {
                        //the new context is not interactive.  So we should set scgui
                        //as the next context before running it.

                        //no longer needed becasue we recycle the current one.  To change it, you run it explicitly.
                        //libsc.SCGUIAppContextSettings uisettings = new libsc.SCGUIAppContextSettings();
                        settings.NextSettings = this.scguiContext.GetSettings().SerializedVersionNothingNext();
                    }
                    libscclient.ContextSettings splaySettingsStack;
                    if (scguiContext.HasParent())
                    {
                        splaySettingsStack = client.GetSettingsStack(client.GetParent(scguiContext),settings);
                    }
                    else
                    {
                        splaySettingsStack = settings;
                    }
                    pedestal.SplayContextStack(splaySettingsStack);
                    
                }
                else if (Control.ModifierKeys == Keys.Alt)
                {
                    //alt key pressed.
                    //this opens the settings for the given context and allows them to
                    //be edited.
	                System.Windows.Forms.Form f = new Form();
                    libpropsguiwf.PropertyPageControl ppc = new libpropsguiwf.PropertyPageControl(launchableSettings.Settings);
	                f.Controls.Add(ppc);
					ppc.Dock = DockStyle.Fill;
                    System.Windows.Forms.Button cancel = new Button();
                    cancel.Text = "&Close";
                    f.Controls.Add(cancel);
                    cancel.Dock = DockStyle.Bottom;
                    f.MaximizeBox = false;
                    f.ShowInTaskbar = false;
                    f.Text = launchableSettings.Settings.NameSpace + ":" + launchableSettings.Settings.Name + " Settings";
                    f.CancelButton = cancel;
	                f.ShowDialog(this);
                    ppc.DisconnectAllEvents();
                }
                else
                {
                    //no relevant modifier keys pressed.  run the selected context in this one's place.
                    libscclient.ContextSettings settings = launchableSettings.Settings;
                    if (launchableSettings.IsIntermediate)
                    {
                        //the new context is not interactive.  So we should set scgui
                        //as the next context before running it.
						
						//not used because we recycle the current one.
                        //libsc.SCGUIAppContextSettings uisettings = new libsc.SCGUIAppContextSettings();
						
                        settings.AppendToEnd(this.SCGuiContext.GetSettings()); 
                    }
                    scguiContext.SetSuccessor(settings);
                    this.Close();
					//System.Windows.Forms.Application.ExitThread();
                    //System.Windows.Forms.Application.Exit();
                    return;
                }
            }
        }

        private void SCGuiWindow_Load(object sender, EventArgs e)
        {

        }

    }
}
